import pickle
from anarci import anarci
import os
import sys
sys.path.insert(0,os.path.dirname(os.path.abspath(__file__)))
from aboss.region_definitions import Accept

a = Accept()
a.numbering_scheme = "imgt"
a.definition = "imgt"

def finding_V_frameworks(numbering, chain):
	"""
	Here we IMGT number germline V genes
	"""
	all_regions = {}
	if chain == "H":
		list_of_frames = ["fwh1", "fwh2", "fwh3", "cdrh1", "cdrh2", "cdrh3"]  
	elif chain == "L":
		list_of_frames = ["fwl1", "fwl2", "fwl3", "cdrl1", "cdrl2", "cdrl3"]
	for frame in list_of_frames:
		all_regions[frame] = {}
		a.set_regions([frame])
		for n in range(len(numbering)):
			if a.accept(numbering[n][0], chain) == 1: 
				if numbering[n][1] == '-':
					continue
				else:
					if numbering[n][0][1] != ' ':
						all_regions[frame][str(numbering[n][0][0]) + numbering[n][0][1]] = numbering[n][1]					
					else:
						if frame == "cdrh3" or frame == "cdrl3":
							if numbering[n][0][0] > 110:
								continue
						all_regions[frame][str(numbering[n][0][0])] = numbering[n][1]
	return all_regions

def finding_J_frameworks(numbering, chain):
	"""
	Here we IMGT number germline V genes
	"""
	all_regions = {}
	if chain == "H":
		list_of_frames = ["cdrh3","fwh4"] 
	elif chain == "L":
		list_of_frames = ["cdrl3","fwl4"] 
	for frame in list_of_frames:
		all_regions[frame] = {}
		a.set_regions([frame])
		for n in range(len(numbering)):
			if a.accept(numbering[n][0], chain) == 1: 
				if numbering[n][1] == '-':
						continue
				else:
					if numbering[n][0][1] != ' ':
						all_regions[frame][str(numbering[n][0][0]) + numbering[n][0][1]] = numbering[n][1]					
					else:
						if frame == "cdrh3" or frame == "cdrl3":
							if numbering[n][0][0] < 115:
								continue
						all_regions[frame][str(numbering[n][0][0])] = numbering[n][1]
	return all_regions

def anarci_fun(list_of_inputs):
	"""
	We use ANARCI to number IMGT germlines
	"""
	function = []
	for n in range(len(list_of_inputs)):
		function.append(("scFv"+str(n), list_of_inputs[n]))
	output = anarci(function, scheme='imgt')
	return output

def remove_dashes(dictionary):
	return { gene : dictionary[gene].replace("-","") for gene in dictionary  } 


def adding_zeros(dictionary):
	amino_acids = sorted(list("QWERTYIPASDFGHKLCVNM"))
	for frame in dictionary:
		for pos in dictionary[frame]:
			for amino in amino_acids:
				if amino not in dictionary[frame][pos]:
					dictionary[frame][pos][amino] = 0
	return dictionary

def createIMGTReference(species, chain,observed_V_gene, observed_J_gene ):
	"""Function that takes species, antibody chain, and lists of observed V and J genes to create IMGT reference matrix
	Inputs:
		@species = species of antibody Igseq data
		@chain = Igseq data antibody chain
		@observed_V_gene = List of V genes found in the Ig-seq data
		@observed_J_gene = List of J genes found in the Ig-seq data

	Return:
		@chain_reference: IMGT reference
	
	"""
	from anarci.germlines import all_germlines as IMGT_germlines

	amino_acids = sorted(list("QWERTYIPASDFGHKLCVNM"))
	chain_reference = {}
	#################################################
	### Loading V and J genes seen in Ig-seq data ###
	#################################################

	list_of_V_germlines = list(set([x for x in observed_V_gene if x != None]))
	list_of_J_germlines = list(set([x for x in observed_J_gene if x != None]))
	
	#########################################################################
	### Collecting amino acid sequences of IMGT genes seen in Ig-seq data ###
	#########################################################################
	if chain == "H":
		IMGT_germline_V_species = remove_dashes(IMGT_germlines['V']['H'][species]) 
		IMGT_germline_J_species = remove_dashes(IMGT_germlines['J']['H'][species])
	elif chain == "L":
		IMGT_germline_V_species = {}
		IMGT_germline_J_species = {}
		IMGT_germline_V_lambda = remove_dashes(IMGT_germlines['V']['L'][species]) 
		IMGT_germline_J_lambda = remove_dashes(IMGT_germlines['J']['L'][species])
		IMGT_germline_V_kappa = remove_dashes(IMGT_germlines['V']['K'][species]) 
		IMGT_germline_J_kappa = remove_dashes(IMGT_germlines['J']['K'][species])
		IMGT_germline_V_species.update(IMGT_germline_V_lambda)
		IMGT_germline_V_species.update(IMGT_germline_V_kappa)
		IMGT_germline_J_species.update(IMGT_germline_J_lambda)
		IMGT_germline_J_species.update(IMGT_germline_J_kappa)	

	#####################################################################################
	### Here we iterate through V and J genes, and recreate a full-length ab sequence ###
	### which is required for ANARCI numbering                                        ###
	#####################################################################################	
	for germ_seq in list_of_V_germlines:
		if germ_seq not in IMGT_germline_V_species:
			continue
		if chain == "H":
			if species in ["human", "mouse", "rabbit", "pig", "rat"]:
				gene_V_sequence = IMGT_germline_V_species[germ_seq] +"A"+ IMGT_germline_J_species['IGHJ1*01']
			elif species in ["alpace"]:
				gene_V_sequence = IMGT_germline_V_species[germ_seq] +"A"+ IMGT_germline_J_species['IGHJ2*01']
			elif species in ["rhesus"]:
				gene_V_sequence = IMGT_germline_V_species[germ_seq] +"A"+ IMGT_germline_J_species['IGHJ5-1*01']
		if chain == "L":
			if species in ["human", "mouse", "rhesus", "pig", "rat"]:
				gene_V_sequence = IMGT_germline_V_species[germ_seq] +"A"+ IMGT_germline_J_species["IGKJ1*01"]
			elif species in ["rabbit"]:
				gene_V_sequence = IMGT_germline_V_species[germ_seq] +"A"+ IMGT_germline_J_species["IGKJ1-1*03"]		
		output = anarci_fun([gene_V_sequence])
		if not output[0][0]:
			continue
		else:
			numbering = output[0][0][0][0]
			aa = finding_V_frameworks(numbering, chain)
			for framework in aa:
				if framework not in chain_reference:
					chain_reference[framework] = {}
				for pos in aa[framework]:
					for ele in aa[framework][pos]:
						if pos not in chain_reference[framework]:
							chain_reference[framework][pos] = {}
						if ele in chain_reference[framework][pos]:
							chain_reference[framework][pos][ele] += 1
						elif ele not in chain_reference[framework][pos]:
							chain_reference[framework][pos][ele] = 1
	for germ_seq in list_of_J_germlines:
		if germ_seq not in IMGT_germline_J_species:
			continue
		germ_J_sequence = IMGT_germline_J_species[germ_seq]
 		if chain == "H":
			if species == "human":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV1-18*01"]
			elif species == "mouse":
				most_common_V_germ_sequence = IMGT_germline_V_species['IGHV1-12*01']
			elif species == "alpaca":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV2S1*01"]
			elif species == "rabbit":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV1S1*01"]
			elif species == "rhesus":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV4-2*01"]
			elif species == "pig":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV1-11*01"]
			elif species == "rat":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGHV2S18*01"]
		if chain == "L":
			if species == "human":
				most_common_V_germ_sequence = IMGT_germline_V_species["IGKV1-12*01"]
			elif species == "mouse":
				most_common_V_germ_sequence = IMGT_germline_V_species['IGLV1*01']
			elif species == "rat":
				most_common_V_germ_sequence = IMGT_germline_V_species['IGKV10S6*01']
			elif species == "rabbit":
				most_common_V_germ_sequence = IMGT_germline_V_species['IGKV1S2*01']	
			elif species == "pig":
				most_common_V_germ_sequence = IMGT_germline_V_species['IGKV1-11*01']	
		complete_seq = most_common_V_germ_sequence + "A" + germ_J_sequence
		output = anarci_fun([complete_seq])
		if not output[0][0]:
			continue
		else:
			numbering = output[0][0][0][0]	
			aa = finding_J_frameworks(numbering, chain)
			for framework in aa:
				if framework not in chain_reference:
					chain_reference[framework] = {}	
				for pos in aa[framework]:
					for ele in aa[framework][pos]:
						if pos not in chain_reference[framework]:
							chain_reference[framework][pos] = {}
						if ele in chain_reference[framework][pos]:
							chain_reference[framework][pos][ele] += 1
						elif ele not in chain_reference[framework][pos]:
							chain_reference[framework][pos][ele] = 1
	chain_reference = adding_zeros(chain_reference)

	return chain_reference

if __name__ == '__main__':
	pass
