import cPickle as pickle
from os.path import join
import os, sys
import multiprocessing as mp
import time
import json
import csv
csv.field_size_limit(sys.maxsize)

def merge_dicts(*dict_args):
    result = {}
    for dictionary in dict_args:
        result.update(dictionary)
    return result

def csv_length():
	"""
	Function that returns the number of non-redundant sequences that parsed ANARCI
	"""
	with open(join("Temp_CSV","temporary.csv")) as csvDataFile:
		csvReader = csv.reader(csvDataFile)
		row_count = sum(1 for row in csvReader)
	return row_count

def read_csv(chunk,chunk_size):
	"""Function that reads the temporary CSV file in chunks of chunk_size
	Inputs:
		@chunk: # of chunks
		@chunk_size: Size of chunks. Default is 250,000 sequences
	Return:
		@csv_dictonary: Python dictionary, where key is antibody sequence, and value is the numbered antibody residues
	"""
	csv_dictonary = {}
	count = 0
	start = chunk*chunk_size

	with open(join("Temp_CSV","temporary.csv")) as csvDataFile:
		csvReader = csv.reader(csvDataFile)
		for row in csvReader:
			if count >= start:
				csv_dictonary[row[0]] = row[4]
			if len(csv_dictonary) == chunk_size:
				return csv_dictonary
			count += 1	
	return csv_dictonary



def check_chain_correctness(pos_num, amino, ReferenceChain):
	"""Function that check if a given amino/position is present in the reference matrix. (0 means it is absent)
	Inputs:
		@pos_num: Position in antibody chain
		@amino: amino acid (or residue)
		@ReferenceChain: Reference matrix
	Return:
		True or None if a amino/position is present in the reference matrix

	"""
	if pos_num in ReferenceChain:
		if ReferenceChain[pos_num][amino] != 0:
			return True
		else:
			return None
	else:
		return None


def checking_sequence_correctness(numbers,reference):
	"""Function runs every residue/position in numbered antibody sequence against the reference matrix. If a residue/position not found in the refence matrix it is flagged.
	Inputs:
		@numbers:   Numbered antibody residue/positions separated by antibody region (e.g. fwh1)
		@reference: Reference Matrix
	Return:
		@current_error: Number of ABOSS flagged residue/positions in sequence
		@incorrect_positions: ABOSS flagged residue/positions in sequence
	"""
	incorrect_positions = []
	current_error = 0
	for frame in numbers:
		for pos_num in numbers[frame]:
			if numbers[frame][pos_num] == "-":
				continue
			
			else:
				if int(pos_num[:3]) < 27:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["f1"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) < 39:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["c1"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) < 56:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["f2"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) < 66:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["c2"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) < 105:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["f3"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) < 118:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["c3"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))

				elif int(pos_num[:3]) >= 118:
					if check_chain_correctness(pos_num, numbers[frame][pos_num], reference["f4"]) == True:
						pass
					else:
						current_error +=1
						incorrect_positions.append((pos_num, numbers[frame][pos_num]))
	if incorrect_positions == []:
		incorrect_positions.append(None)

	return current_error, incorrect_positions

def initiating_ref_matrix(passed_new_lines, reference_matrix,core):
	"""Passing numbered antibody sequences to a function that ABOSS flags antibody sequences
	Inputs:
		@passed_new_lines: python dictinary where key is antibody sequence, and value is the numbered antibody residues
		@reference_matrix: Reference matrix build by ABOSS using the residue error rate
		@core: core number to track the progress
	Return:
		@ABOSSParsedSequences: Sequences with ABOSS flagged reisdue/positions
	"""
	all_regions_aa = {}
	ABOSSParsedSequences = {}
	for nonRedundantSequence in passed_new_lines:
		
		aa = json.loads(passed_new_lines[nonRedundantSequence])


		error, incorrect_residues = checking_sequence_correctness(aa,reference_matrix)  # Here I pass sequence to a reference matrix
		if error not in ABOSSParsedSequences:
			ABOSSParsedSequences[error]= {}
			all_regions_aa[error] = {}
		if nonRedundantSequence in ABOSSParsedSequences[error]:
			continue
		else:
			if type(originalDataset[nonRedundantSequence]) == int:
				multiplication_factor = originalDataset[nonRedundantSequence]
			else:
				multiplication_factor = len(originalDataset[nonRedundantSequence])
			ABOSSParsedSequences[error][nonRedundantSequence] = (multiplication_factor, incorrect_residues)
	print "Portion %d has finished" %(core+1)

	return ABOSSParsedSequences


def flagResiduePositions(cores, startingData,antibodychain):
	""" Function that goes through all sequences in Ig-Seq data, and checks if residue/positions are above the residue error rate.
	Inputs:
		@cores: number of cores used in aboss analysis
		@startingData: python dictionary,key is antibody sequence, values corresponding redundancy
		@antibodychain: H or L
	Outputs:
		@all_regions_aa: Counts of amino acids at antibody positions in Ig-seq data separated by the number of ABOSS flagged residue/positions
		@correct_sequences: Original antibody sequences separated by the number of ABOSS flagged residue/positions
	"""
	global originalDataset
	global chain
	
	originalDataset = startingData
	chain = antibodychain

	reference_matrix = {}
	if chain == "H":
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwh1"), "rb") as m:
			reference_matrix["f1"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwh2"), "rb") as m:
			reference_matrix["f2"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwh3"), "rb") as m:
			reference_matrix["f3"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwh4"), "rb") as m:
			reference_matrix["f4"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrh1"), "rb") as m:
			reference_matrix["c1"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrh2"), "rb") as m:
			reference_matrix["c2"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrh3"), "rb") as m:
			reference_matrix["c3"] = pickle.load(m).transpose()

	if chain == "L":
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwl1"), "rb") as m:
			reference_matrix["f1"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwl2"), "rb") as m:
			reference_matrix["f2"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwl3"), "rb") as m:
			reference_matrix["f3"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("fwl4"), "rb") as m:
			reference_matrix["f4"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrl1"), "rb") as m:
			reference_matrix["c1"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrl2"), "rb") as m:
			reference_matrix["c2"] = pickle.load(m).transpose()
		with open("Reference_matrix/reference_matrix_%s.pkl" % ("cdrl3"), "rb") as m:
			reference_matrix["c3"] = pickle.load(m).transpose()

	correct_sequences = {}
	count = 0
	all_regions_aa = {}
	##############################################################################
	# Here I need to add a for loop that collect all parsed_reads_raw_num_%d.pkl #
	##############################################################################
	t1 = time.time()
	temp_csv_length = csv_length()
	chunk_size = 250000
	number_of_chunks = temp_csv_length/ chunk_size +1
	for chunk in range(number_of_chunks):
		chunkNumberedSequences = read_csv(chunk,chunk_size)
		print "The current analysed chunk is %d of out %d" %(int(chunk)+1,number_of_chunks)
		num_of_entries = len(chunkNumberedSequences)
		print "The total number of non-redundant sequences in the chunk is %d" %(num_of_entries)

		#################################
		## ABOSS is parallelized now ####
		#################################

		pool = mp.Pool(processes=cores)
		items_sort = chunkNumberedSequences.items()
		results = [ pool.apply_async(initiating_ref_matrix, args = (dict(items_sort[i::cores]), reference_matrix,i)) for i in xrange(cores) ]
		del items_sort
		filtered_sequences_portion = [ r.get() for r in results ]
		pool.close()
		del results			
		for portion in range(len(filtered_sequences_portion)):

			for error in filtered_sequences_portion[portion]:
				if error not in all_regions_aa:
					all_regions_aa[error] = {}
					correct_sequences[error] = {}

				correct_sequences[error] = merge_dicts(correct_sequences[error],filtered_sequences_portion[portion][error])

				for sequence in filtered_sequences_portion[portion][error]:
					aa = json.loads(chunkNumberedSequences[sequence])

					if type(originalDataset[sequence]) == int:
						multiplication_factor = originalDataset[sequence]
					else:
						multiplication_factor = len(originalDataset[sequence])
					for framework in aa:
						if framework not in all_regions_aa[error]:
							all_regions_aa[error][framework] = {}
						for pos in aa[framework]:
							for ele in aa[framework][pos]:
								if pos not in all_regions_aa[error][framework]:
									all_regions_aa[error][framework][pos] = {}
								if ele in all_regions_aa[error][framework][pos]:
									all_regions_aa[error][framework][pos][ele] += multiplication_factor
								elif ele not in all_regions_aa[error][framework][pos]:
									all_regions_aa[error][framework][pos][ele] = multiplication_factor
		print "Time took to analyse this file: ",time.time() - t1

		del filtered_sequences_portion
		del chunkNumberedSequences

	return all_regions_aa,correct_sequences

if __name__ == '__main__':
	pass
