import pandas as pd
import pickle
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import os
from os.path import join
import sys
from aboss.reference_matrix import calculate_posCount, cdrh3,frame_canon

def control_bar(frame, regions_aaCount, pos_Count):
	""" Function to calcualte the average amino acid distribution and entropies in antibody regions
	Inputs:
		@frame: antibody region e.g. cdrh1
		@regions_aaCount: Counts of amino acid residues at positions in ABOSS filtered Ig-seq Data
	Return:
		@average_AAUsage: Average amino acid usage in the given antibody region
		@entropy_contr:   Average entropy of positions in the given antibody region
	"""
	length_of_frame = 0
	entropy = 0
	entropy_contr = {}
	control_frame_prop = {frame:{}}
	average_AAUsage = {frame:{}}

	length_of_frame = sum([pos_Count[pos] for pos in pos_Count])

	for pos in regions_aaCount:
		for ele in regions_aaCount[pos]:
			if ele not in control_frame_prop[frame]:
				control_frame_prop[frame][ele] = regions_aaCount[pos][ele]
			else:
				control_frame_prop[frame][ele] += regions_aaCount[pos][ele]
	for ele in control_frame_prop[frame]:
		average_AAUsage[frame][ele] = control_frame_prop[frame][ele] / float(length_of_frame)
		if average_AAUsage[frame][ele] > 0:
           		entropy += - average_AAUsage[frame][ele]*math.log(average_AAUsage[frame][ele], 2)
	entropy_contr[frame] = entropy
	return average_AAUsage, entropy_contr

def adding_zeros(dictionary):
	amino_acids = sorted(list("QWERTYIPASDFGHKLCVNM"))
	for pos in dictionary:
		for amino in amino_acids:
			if amino not in dictionary[pos]:
				dictionary[pos][amino] = 0
	return dictionary

def plotGraps(frame,regions_aaCount):
	"""Plotting three types of graphs here:
		1) Amino acid usage by position in ABOSS filtered Ig-seq Data
		2) Entropy in antibody regions in ABOSS filtered Ig-seq Data
		3) Counts of amino acid entries at positions in ABOSS filtered Ig-seq Data
	Inputs:
		@frame: Analysed antibody region such as cdrh3
		@regions_aaCount: Counts of amino acid residues at positions in ABOSS filtered Ig-seq Data 
	"""

	regions_aaCount = regions_aaCount[0][frame]
	entropy_dict = {frame:{}}
	#################################
	# Here we create a Graph folder #
	#################################

	if os.path.isdir("Graphs") != True:
		os.makedirs("Graphs")

	regions_aaCount = adding_zeros(regions_aaCount)
	pos_Count = calculate_posCount(regions_aaCount)
	weighted_contr, _ = control_bar(frame, regions_aaCount, pos_Count)
	##############################################
	### Here we calculate entropy by position ####
	##############################################
	for pos in regions_aaCount:
		entropy = 0
		for ele in regions_aaCount[pos]:
			 regions_aaCount[pos][ele] = float(regions_aaCount[pos][ele]) / pos_Count[pos]  # Amino acid distribution by position
			 if regions_aaCount[pos][ele] > 0:
   				 entropy += -regions_aaCount[pos][ele]*math.log(regions_aaCount[pos][ele], 2) # Entropy by position
		entropy_dict[frame][pos] = entropy
	#########################################################
	if weighted_contr == {frame: {}}:
		print "The input files didn't contain sequence of ", frame
	else:	
		df_entropy = pd.DataFrame(entropy_dict[frame], index = [0]).fillna(0)  # Dataframe of position entropies
		df_aa_proportion = pd.DataFrame(regions_aaCount).fillna(0)	       # Dataframe of amino acid distribution by position
		df_pos_len = pd.DataFrame(pos_Count, index=[0]).fillna(0) 	       # Dataframe of Number of entries by position
		###################################################
		##### Here we put positions in the IMGT order #####
		###################################################
		cols = df_aa_proportion.columns.tolist()
		if frame not in ["cdrh3","cdrl3"]:
		#	print frame
			new_cols = frame_canon(cols)
		else:
			new_cols = cdrh3(cols)	
		#print new_cols
		###############################
		##### ENTROPY PLOTTING  #######
		###############################
		entropy_complete_plot = df_entropy[new_cols].transpose()

		#entropy_control_bar = pd.DataFrame(entropy_contr, index = [0]).fillna(0)  # entropy_control
		#entropy_control_bar = entropy_control_bar.transpose()
		#entropy_complete_plot = df_entropy.append(entropy_control_bar)

		########################################
		### Graph for entries at positions #####
		########################################
		df_pos_len = df_pos_len[new_cols].transpose()

		###########################################
		##### AA distribution by position plot ####
		###########################################
		df_aa_proportion = df_aa_proportion[new_cols].transpose()

		averaged_aa_control_bar = pd.DataFrame(weighted_contr).fillna(0) 
		averaged_aa_control_bar = averaged_aa_control_bar.transpose()	
		aa_proportion_complete_plot = df_aa_proportion.append(averaged_aa_control_bar)

		###### Plotting starts here########
		### entropy by position ###
		fig, axis = plt.subplots()
		entropy_complete_plot.plot.bar(title ="%s entropy, ABOSS filtered data" %(frame), legend = False, color="red")
		fig.set_size_inches(7.5,4.5, forward=True)
		plt.savefig("Graphs/%s entropy ABOSS filtered data" %(frame), dpi= 300)
		plt.clf()
		### amino acids by position ###
		colors_dict = dict([("A", "#a2a0a5"),("C", "#fff319"),("D","#22a56f"),("E","#19ff34"),("F","#031dc1"),("G","#35ffee"),("H","#6e728c"),("I","#057200"), ("K","#720070"),("L","#914b00"),("M","#847806"),("N","#f24d76"),("P","#fcc7ed"),("Q","#cc0c0c"),("R","#2cc124"),("S","#c124c1"),("T","#ff6600"),("V","#c6e833"),("W","#ff0000"),("Y","#6fa522")])
		amino_acid_order =  list(aa_proportion_complete_plot)
		my_colors = [colors_dict[x] for x in amino_acid_order]
		fig, axis = plt.subplots()
		aa_proportion_complete_plot.plot.bar(stacked=True, color = my_colors, title =" %s proportions, ABOSS filtered data" %(frame), legend='reverse').legend(bbox_to_anchor=(0.99, 1.02))
		fig.set_size_inches(9,4.5, forward=True)
		plt.savefig("Graphs/%s proportion ABOSS filtered data" %(frame),dpi= 300)
		plt.clf()
		### entries by postion ###
		fig, axis = plt.subplots()
		df_pos_len.plot.bar(color="green", legend = False, title=" %s entries, ABOSS filtered data" %(frame))
		fig.set_size_inches(7.5,4.5, forward=True)
		plt.savefig("Graphs/%s entries ABOSS filtered data" %(frame), dpi= 300)
		plt.clf()
		plt.close('all')
		return "The analysis of %s domain has finished" % (frame)

if __name__ == '__main__':
	pass
