#!/usr/bin/env python

from distutils.core import setup
import distutils

#                     		           #
# ANARCI was developed by James Dunbar     #
# Maintained by members of OPIG 	   #
#                               	   #
#We check if an user has an preinstalled version of ANARCI
try:
	import anarci
	anarci_status="PreInstalled"
except ImportError:
	anarci_status = "Clean"

setup(name='aboss',
      version='1.01',
      description='Filtering Ig-seq with structural information',
      author='Aleksandr Kovaltsuk',
      author_email='opig@stats.ox.ac.uk',
      url='http://opig.stats.ox.ac.uk//resources',
      packages=["aboss",'anarci', 'anarci.Bio', 'anarci.Bio.Align',
      'anarci.Bio.Alphabet', 'anarci.Bio.Data', 'anarci.Bio._py3k',
      'anarci.Bio.SearchIO','anarci.Bio.SearchIO._model', 'anarci.Bio.SeqIO', 'anarci.Bio.SearchIO.HmmerIO', 'anarci.Bio.AlignIO'  ],
      package_dir={"aboss":"lib/python/aboss",
		   'anarci': 'lib/python/anarci',
                   'anarci.Bio.AlignIO': 'lib/python/anarci/Bio/AlignIO', 
                   'anarci.Bio.SearchIO': 'lib/python/anarci/Bio/SearchIO', 
                   'anarci.Bio.SearchIO.HmmerIO': 'lib/python/anarci/Bio/SearchIO/HmmerIO', 
                   'anarci.Bio.Alphabet': 'lib/python/anarci/Bio/Alphabet', 
                   'anarci.Bio._py3k': 'lib/python/anarci/Bio/_py3k', 
                   'anarci.Bio.Data': 'lib/python/anarci/Bio/Data', 
                   'anarci.Bio': 'lib/python/anarci/Bio', 
                   'anarci.Bio.SearchIO._model': 'lib/python/anarci/Bio/SearchIO/_model', 
                   'anarci.Bio.SeqIO': 'lib/python/anarci/Bio/SeqIO',
                   'anarci.Bio.Align': 'lib/python/anarci/Bio/Align'},
      scripts=['bin/ANARCI', "bin/muscle","bin/ABOSS"],
      license="MIT"
     )
	
import sys
if sys.argv[1] != "install":
    sys.exit(0)
try:
	import aboss
except ImportError:
	print "The aboss package was put in the directory, which is not in your PATH variable! Please add the path, where ABOSS and ANARCI were put, to your PATH variable :)"
	sys.exit(1)

import anarci
import shutil,os,subprocess

print "\n\n\tABOSS code is found in: {0}".format(os.path.dirname(os.path.abspath(aboss.__file__) ))

print "\tANARCI code is found in: {0}".format(os.path.dirname(anarci.__file__))

os.chdir("build_pipeline")
proc = subprocess.Popen(["bash", "RUN_pipeline.sh"], stdout = subprocess.PIPE, stderr = subprocess.PIPE)
o, e = proc.communicate()

if os.path.isfile(os.path.join(os.path.dirname(anarci.__file__),"germlines.py")) != True:
    shutil.copy( "curated_alignments/germlines.py", os.path.dirname(anarci.__file__) )

shutil.copy( "curated_alignments/germlines.py", os.path.dirname(aboss.__file__) )

if anarci_status=="PreInstalled":
    print "\n\tThe Preinstalled Version of ANARCI is found\n\tYOUR current HMMs will not be overwritten"
else:
    from time import sleep
    print "\n\tSince ANARCI was not initially installed, creating HMMs"
    sleep(2)
    assert os.path.isdir(os.path.join(os.path.dirname( anarci.__file__ ), "dat/")) == False, "The HMMs already exist"
#    distutils.dir_util.copy_tree( "HMMs", os.path.join(os.path.dirname( anarci.__file__ ), "dat/HMMs/") )
    shutil.copytree("HMMs",os.path.join(os.path.dirname( anarci.__file__ ), "dat/HMMs/") )
    print "\t HMMs have been created and found in {0}".format(os.path.join(os.path.dirname( anarci.__file__ ), "dat/HMMs/"))
try:
    pr = subprocess.Popen( [ "ABOSS","-h" ], stdout=subprocess.PIPE, stderr=subprocess.PIPE )
    o, e = pr.communicate()

    print "\n\tYou have successfully installed ABOSS version: {0}\n\tThank you for downloading ABOSS\n".format(aboss.__version__)
    print "\n\tCheck Format_examples directory for file formats that ABOSS accepts"
except OSError:
    print "ABOSS was put in the bin directory, which is not in your PATH variable!, Please alter your PATH variable accordingly"


